/*==============================================================*/
/* Database name:  engine                                       */
/* DBMS name:      MySQL 3.23                                   */
/* Created on:     02.03.2005 13:44:39                          */
/*==============================================================*/


drop table if exists acl;

drop table if exists aevent_log;

drop table if exists asession;

drop table if exists auser;

drop table if exists auser_to_auser;

/*==============================================================*/
/* Table: acl                                                   */
/*==============================================================*/
/*     */
create table if not exists acl
(
   object_id                      INTEGER UNSIGNED               not null default 0,
   auser_id                       INTEGER UNSIGNED               not null default 0,
   rights                         INTEGER UNSIGNED               not null default 0,
   unique (auser_id, object_id)
);

/* object_id -    */;

/* rights -   -   ,  -  . */;

/*==============================================================*/
/* Table: aevent_log                                            */
/*==============================================================*/
/*  */
create table if not exists aevent_log
(
   aevent_log_id                  INTEGER UNSIGNED               not null AUTO_INCREMENT,
   auser_id                       INTEGER UNSIGNED               not null default 0,
   event_type                     MEDIUMINT                      not null default 256,
   stat                           MEDIUMINT                      not null,
   dt                             DATETIME                       not null,
   content                        VARCHAR(255),
   primary key (aevent_log_id)
);

/* auser_id -     */;

/* event_type -    (  auser) */;

/* stat -  (,    ) */;

/* dt -   */;

/* content -   -  ... */;

/*==============================================================*/
/* Index: ix_aevent_log_0                                       */
/*==============================================================*/
create index ix_aevent_log_0 on aevent_log
(
   auser_id,
   dt
);

/*==============================================================*/
/* Table: asession                                              */
/*==============================================================*/
/* ,     */
create table if not exists asession
(
   asession_id                    INTEGER UNSIGNED               not null AUTO_INCREMENT,
   auser_id                       INTEGER UNSIGNED               not null default 0,
   sid                            VARCHAR(64)                    not null,
   auid                           VARCHAR(64)                    not null,
   dt_access                      DATETIME                       not null,
   dt_logon                       DATETIME,
   dt_logout                      DATETIME,
   primary key (asession_id)
);

/* sid - ,    */;

/* auid - ,    */;

/* dt_access -    */;

/* dt_logon -    */;

/* dt_logout -    */;

/*==============================================================*/
/* Index: ix_asession_0                                         */
/*==============================================================*/
create index ix_asession_0 on asession
(
   auser_id
);

/*==============================================================*/
/* Index: ix_asession_1                                         */
/*==============================================================*/
create index ix_asession_1 on asession
(
   auid,
   dt_access
);

/*==============================================================*/
/* Index: ix_asession_2                                         */
/*==============================================================*/
create index ix_asession_2 on asession
(
   dt_access
);

/*==============================================================*/
/* Table: auser                                                 */
/*==============================================================*/
/*   
       owner (auser_type_id = 3)
     ,      .
   ..  /        , 
      - , - ,    -   .
   event_type -  ,  :
   1 - , 
   2 - , 
   8 -  , 
   4 - request_password, 
   16 -  , 
   32 -  email, 
   64 -   , 
   128 -  , 
   256 -  
   ...    */
create table if not exists auser
(
   auser_id                       INTEGER UNSIGNED               not null AUTO_INCREMENT,
   auser_type_id                  TINYINT UNSIGNED               not null default 0,
   rights                         INTEGER UNSIGNED               not null default 0,
   name                           VARCHAR(127)                   not null,
   description                    TEXT,
   email                          VARCHAR(63)                    not null,
   passwd                         VARCHAR(63)                    not null,
   new_passwd                     VARCHAR(63),
   dt_register                    DATETIME                       not null,
   dt_logon                       DATETIME,
   dt_logout                      DATETIME,
   is_published                   TINYINT UNSIGNED               not null default 1,
   is_default                     TINYINT UNSIGNED               not null default 0,
   connections_limit              TINYINT UNSIGNED               not null default 1,
   event_type                     MEDIUMINT UNSIGNED             not null default 0,
   primary key (auser_id),
   unique (auser_type_id, name)
);

/* auser_type_id -   (0 - , 1 - ) */;

/* rights -  /    */;

/* name -    */;

/* description -  (  ) */;

/* email - email  */;

/* passwd -  */;

/* new_passwd -   ( ,      ) */;

/* dt_register - /  */;

/* dt_logon - /   */;

/* dt_logout - /   */;

/* is_published -     */;

/* is_default -  .    1       */;

/* connections_limit -    */;

/* event_type -  ,     */;

/*==============================================================*/
/* Index: ix_auser_0                                            */
/*==============================================================*/
create index ix_auser_0 on auser
(
   auser_type_id,
   is_published,
   dt_logon
);

/*==============================================================*/
/* Index: ix_auser_1                                            */
/*==============================================================*/
create index ix_auser_1 on auser
(
   dt_register
);

/*==============================================================*/
/* Index: ix_auser_2                                            */
/*==============================================================*/
create index ix_auser_2 on auser
(
   auser_type_id,
   name,
   passwd
);

/*==============================================================*/
/* Index: ix_auser_3                                            */
/*==============================================================*/
create index ix_auser_3 on auser
(
   auser_type_id,
   email,
   name
);

/*==============================================================*/
/* Table: auser_to_auser                                        */
/*==============================================================*/
/*   
   
   rights (   ):
   0 - User
   1 - Supervisory (     )
   2 - Remove self ( ?) */
create table if not exists auser_to_auser
(
   auser_id                       INTEGER UNSIGNED               not null default 0,
   parent_id                      INTEGER UNSIGNED               not null default 0,
   rights                         INTEGER UNSIGNED               not null default 0
);

/* auser_id -  (  ) */;

/* parent_id -     (  ) */;

/* rights - ,       */;

/*==============================================================*/
/* Index: ix_auser_to_auser_0                                   */
/*==============================================================*/
create unique index ix_auser_to_auser_0 on auser_to_auser
(
   auser_id,
   parent_id
);



/*==============================================================*/
/* Insert default data		                                    */
/*==============================================================*/
INSERT INTO auser (auser_id, name, passwd, email, description, auser_type_id, rights, is_published, dt_register) VALUES (1, 'Owner', 'onwner', 'owner', ' ', 2, 16777215, 1, NOW());
INSERT INTO auser (auser_id, name, passwd, email, description, auser_type_id, rights, is_published, dt_register) VALUES (2, 'Admins', 'group', 'group', '  ', 1, 16777215, 1, NOW());
INSERT INTO auser (auser_id, name, passwd, email, description, auser_type_id, rights, is_published, dt_register) VALUES (3, 'admin', 'admin', 'misha@design.ru', '', 0, 16777215, 1, NOW());
INSERT INTO auser_to_auser (auser_id, parent_id) VALUES (3,2);
