/*==============================================================*/
/* DBMS name:      PostgreSQL 7.3.2                                   */
/* Created on:     13.06.2003                          */
/*==============================================================*/

/*==============================================================*/
/* Table: acl                                                   */
/*==============================================================*/
/*     */

CREATE TABLE acl
(
   object_id                      INT                        NOT NULL DEFAULT 0,
   auser_id                       INT                        NOT NULL DEFAULT 0,
   rights                         INT                        NOT NULL DEFAULT 0
);


/* object_id -    */;

/* rights -   -   ,  -  . */;

/*==============================================================*/
/* Index: ix_acl_0                                   */
/*==============================================================*/
CREATE UNIQUE INDEX ix_acl_0 ON acl
(
   object_id,
   auser_id
);


/*==============================================================*/
/* Table: aevent_log                                            */
/*==============================================================*/
CREATE TABLE aevent_log
(
	aevent_log_id SERIAL NOT NULL,
	auser_id INT NOT NULL DEFAULT 0,
	event_type INT NOT NULL DEFAULT 256,
	stat INT NOT NULL,
	dt TIMESTAMP NOT NULL,
	content VARCHAR(255),
	PRIMARY KEY (aevent_log_id)
);

/* auser_id -     */;

/* event_type -    (  auser) */;

/* stat -  (,    ) */;

/* dt -   */;

/* content -   -  ... */;

/*==============================================================*/
/* Index: ix_aevent_log_0                                       */
/*==============================================================*/
CREATE INDEX ix_aevent_log_0 ON aevent_log
(
   auser_id,
   dt
);

/*==============================================================*/
/* Table: asession                                              */
/*==============================================================*/
/* ,     */
CREATE TABLE asession
(
	asession_id SERIAL NOT NULL,
	auser_id INT NOT NULL DEFAULT 0,
	sid VARCHAR(64) NOT NULL,
	auid VARCHAR(64) NOT NULL,
	dt_access TIMESTAMP NOT NULL,
	dt_logon TIMESTAMP,
	dt_logout TIMESTAMP,
	PRIMARY KEY (asession_id)
);

/* sid - ,    */;

/* auid - ,    */;

/* dt_access -    */;

/* dt_logon -    */;

/* dt_logout -    */;

/*==============================================================*/
/* Index: ix_asession_0                                         */
/*==============================================================*/
CREATE INDEX ix_asession_0 ON asession
(
   auser_id
);

/*==============================================================*/
/* Index: ix_asession_1                                         */
/*==============================================================*/
CREATE INDEX ix_asession_1 ON asession
(
   auid,
   dt_access
);

/*==============================================================*/
/* Index: ix_asession_2                                         */
/*==============================================================*/
CREATE INDEX ix_asession_2 ON asession
(
   dt_access
);

/*==============================================================*/
/* Table: auser                                                 */
/*==============================================================*/
/*   
       owner (auser_type_id = 3)
     ,      .
   ..  /        , 
      - , - ,    -   .
   event_type -  ,  :
   1 - , 
   2 - , 
   8 -  , 
   4 - request_password, 
   16 -  , 
   32 -  email, 
   64 -   , 
   128 -  , 
   256 -  
   ...    */
CREATE TABLE auser
(
	auser_id SERIAL NOT NULL,
	auser_type_id INT NOT NULL DEFAULT 0,
	rights INT NOT NULL DEFAULT 0,
	name VARCHAR(127) NOT NULL,
	description TEXT,
	email VARCHAR(63) NOT NULL,
	passwd VARCHAR(63) NOT NULL,
	new_passwd VARCHAR(63),
	dt_register TIMESTAMP NOT NULL,
	dt_logon TIMESTAMP,
	dt_logout TIMESTAMP,
	is_published INT NOT NULL DEFAULT 1,
	is_default INT NOT NULL DEFAULT 0,
	connections_limit INT NOT NULL DEFAULT 1,
	event_type INT NOT NULL DEFAULT 0,
	PRIMARY KEY (auser_id),
	UNIQUE (auser_type_id, name)
);

/* auser_type_id -   (0 - , 1 - ) */;

/* rights -  /    */;

/* name -    */;

/* description -  (  ) */;

/* email - email  */;

/* passwd -  */;

/* new_passwd -   ( ,      ) */;

/* dt_register - /  */;

/* dt_logon - /   */;

/* dt_logout - /   */;

/* is_published -     */;

/* is_default -  .    1       */;

/* connections_limit -    */;

/* event_type -  ,     */;

/*==============================================================*/
/* Index: ix_auser_0                                            */
/*==============================================================*/
CREATE INDEX ix_auser_0 ON auser
(
   auser_type_id,
   is_published,
   dt_logon
);

/*==============================================================*/
/* Index: ix_auser_1                                            */
/*==============================================================*/
CREATE INDEX ix_auser_1 ON auser
(
   dt_register
);

/*==============================================================*/
/* Index: ix_auser_2                                            */
/*==============================================================*/
CREATE INDEX ix_auser_2 ON auser
(
   auser_type_id,
   name,
   passwd
);

/*==============================================================*/
/* Index: ix_auser_3                                            */
/*==============================================================*/
CREATE INDEX ix_auser_3 ON auser
(
   auser_type_id,
   email,
   name
);


/*==============================================================*/
/* Table: auser_to_auser                                        */
/*==============================================================*/
/*    */
CREATE TABLE auser_to_auser
(
	auser_id INT NOT NULL DEFAULT 0,
	parent_id INT NOT NULL DEFAULT 0,
	rights INT NOT NULL DEFAULT 0
);
/* auser_id -  (  ) */;

/* parent_id -     (  ) */;

/* rights - ,       */;


/*==============================================================*/
/* Index: ix_auser_to_auser_0                                   */
/*==============================================================*/
CREATE UNIQUE INDEX ix_auser_to_auser_0 ON auser_to_auser
(
   auser_id,
   parent_id
);



/*==============================================================*/
/* Insert DEFAULT data		                                    */
/*==============================================================*/
INSERT INTO auser (auser_id, name, passwd, email, description, auser_type_id, rights, is_published, dt_register) VALUES (1, 'Owner', 'onwner', 'owner', ' ', 2, 16777215, 1, CURRENT_TIMESTAMP);
INSERT INTO auser (auser_id, name, passwd, email, description, auser_type_id, rights, is_published, dt_register) VALUES (2, 'Admins', 'group', 'group', '  ', 1, 16777215, 1, CURRENT_TIMESTAMP);
INSERT INTO auser (auser_id, name, passwd, email, description, auser_type_id, rights, is_published, dt_register) VALUES (3, 'admin', 'admin', 'misha@design.ru', '', 0, 16777215, 1, CURRENT_TIMESTAMP);
INSERT INTO auser_to_auser (auser_id, parent_id) VALUES (3,2);
